#!/usr/bin/python
# -*- coding: utf-8 -*-
import codecs
import json
import random

import time
import urllib

from homevee.Manager.custom_voice_commands import run_custom_voice_commands
from context_keys import *
from homevee.VoiceAssistant import voice_patterns, Modules, helper
import sys

reload(sys)
sys.setdefaultencoding('utf8')

def voice_command_cloud(username, text, user_last_location, room, db):
    # return {'msg_speech': "Der Assistent ist noch nicht erreichbar.", 'msg_text': "Der Assistent ist zur Zeit noch nicht fertig, wird aber gerade implementiert."}

    # room => damit z.b. bei "mach das licht an" das licht im richtigen raum angeht

    # print "voice_command"

    text = text.lower()

    text = helper.replace_voice_commands(text, username, db)

    # run custom voice_commands
    answer = run_custom_voice_commands(text, username, db)

    if (answer is not None):
        return {'msg_speech': answer, 'msg_text': answer}

    print "Voice command: " + codecs.encode(text, 'utf-8')

    context = helper.get_context(username, db)

    if context is not None:
        context_data = json.loads(context['CONTEXT_DATA'])

        print "Kontext: " + str(context_data)

        if context['CONTEXT_KEY'] == SET_MODES:
            answer = Modules.voice_set_modes.set_modes_voice(username, text, context_data, db)
        elif context['CONTEXT_KEY'] == SET_THERMOSTAT:
            answer = Modules.voice_set_modes.set_thermostat(username, text, context_data, db)
        elif context['CONTEXT_KEY'] == CONTROL_BLINDS:
            answer = Modules.voice_set_modes.control_blinds(username, text, context_data, db)
        elif context['CONTEXT_KEY'] == GET_WEATHER:
            answer = Modules.voice_weather.get_weather(username, text, context_data, db)
        elif context['CONTEXT_KEY'] == ADD_NUTRITION:
            answer = Modules.voice_nutrition_manager.voice_add_nutrition_item(username, text, context_data, db)
        elif context['CONTEXT_KEY'] == TV_PROGRAMME:
            answer = Modules.voice_tv.get_tv(username, text, context_data, db)
        elif context['CONTEXT_KEY'] == GET_SENSOR_DATA:
            answer = Modules.voice_get_sensor_data.get_sensor_data(username, text, context_data, db)

        if answer is None or not answer:
            return voice_command_cloud(username, text, user_last_location, room, db)

    else:
        MIN_CONFIDENCE = 0.95

        try:
            result = helper.classify_voice_command(text)
        except:
            return voice_command(username, text, user_last_location, room, db)

        print result

        label = result['label']
        confidence = result['confidence']

        voice_modules = [Modules.voice_calculator, Modules.voice_set_modes, Modules.voice_calendar,
                         Modules.voice_conversation, Modules.voice_get_sensor_data, Modules.voice_tv,
                         Modules.voice_jokes, Modules.voice_rgb_control, Modules.voice_weather,
                         Modules.voice_tv_tipps, Modules.voice_movie_api]

        module_map = {}

        for module in voice_modules:
            module_map[module.get_label()] = module.run_command

        if confidence >= MIN_CONFIDENCE and label in module_map:
            answer = module_map[label](username, text, context, db)
        else:
            answer = Modules.voice_conversation.conversation(username, text, context, db)

    return (answer)

def voice_command(username, text, user_last_location, room, db):
    #return {'msg_speech': "Der Assistent ist noch nicht erreichbar.", 'msg_text': "Der Assistent ist zur Zeit noch nicht fertig, wird aber gerade implementiert."}

    #room => damit z.b. bei "mach das licht an" das licht im richtigen raum angeht

    #print "voice_command"

    text = text.lower()

    try:
        helper.classify_voice_command(text)
    except:
        print "homevee assistant not working"

    text = helper.replace_voice_commands(text, username, db)

    #run custom voice_commands
    answer = run_custom_voice_commands(text, username, db)

    if(answer is not None):
        return {'msg_speech': answer, 'msg_text': answer}

    print "Voice command: "+codecs.encode(text, 'utf-8')

    context = helper.get_context(username, db)

    if context is not None:
        context_data = json.loads(context['CONTEXT_DATA'])

        print "Kontext: "+str(context_data)

        if context['CONTEXT_KEY'] == SET_MODES:
            answer = Modules.voice_set_modes.set_modes_voice(username, text, context_data, db)
        elif context['CONTEXT_KEY'] == SET_THERMOSTAT:
            answer = Modules.voice_set_modes.set_thermostat(username, text, context_data, db)
        elif context['CONTEXT_KEY'] == CONTROL_BLINDS:
            answer = Modules.voice_set_modes.control_blinds(username, text, context_data, db)
        elif context['CONTEXT_KEY'] == GET_WEATHER:
            answer = Modules.voice_weather.get_weather(username, text, context_data, db)
        elif context['CONTEXT_KEY'] == ADD_NUTRITION:
            answer = Modules.voice_nutrition_manager.voice_add_nutrition_item(username, text, context_data, db)
        elif context['CONTEXT_KEY'] == TV_PROGRAMME:
            answer = Modules.voice_tv.get_tv(username, text, context_data, db)
        elif context['CONTEXT_KEY'] == GET_SENSOR_DATA:
            answer = Modules.voice_get_sensor_data.get_sensor_data(username, text, context_data, db)

        if answer is None or not answer:
            return voice_command(username, text, user_last_location, room, db)

    else:
        #Schalter
        if helper.contains_pattern(voice_patterns.GET_PATTERN_SET_MODES(db), text):
            answer = Modules.voice_set_modes.run_command(username, text, context, db)
        #if contains_pattern(voice_patterns.PATTERN_SET_MODES, text):
        #    answer = set_modes_voice(username, text, context, db)
        #Schalter
        elif helper.contains_pattern(voice_patterns.GET_PATTERN_WOL(db), text):
            answer = Modules.voice_wol.voice_wol(username, text, context, db)
        #Thermostat
        elif helper.contains_pattern(voice_patterns.PATTERN_SET_THERMOSTAT, text):
            answer = Modules.voice_set_modes.set_thermostat(username, text, context, db)
        #Rolladensteuerung
        elif helper.contains_pattern(voice_patterns.PATTERN_CONTROL_BLINDS, text):
            answer = Modules.voice_set_modes.control_blinds(username, text, context, db)
        #Dimmer
        elif helper.contains_pattern(voice_patterns.PATTERN_SET_DIMMER, text):
            answer = Modules.voice_set_modes.set_dimmer(username, text, context, db)
        #Einkaufsliste
        elif helper.contains_pattern(voice_patterns.PATTERN_ADD_SHOPPING_LIST, text):
            answer = Modules.voice_shopping_list.add_to_shopping_list(username, text, context, db)
        elif helper.contains_pattern(voice_patterns.PATTERN_GET_SHOPPING_LIST, text):
            answer = Modules.voice_shopping_list.get_shopping_list(username, text, context, db)
        #Wochentag
        elif helper.contains_pattern(voice_patterns.PATTERN_DATE_WEEKDAY, text):
            answer = Modules.voice_get_weekday.run_command(username, text, context, db)
        #Wetter
        elif helper.contains_pattern(voice_patterns.PATTERN_WEATHER, text):
            answer = Modules.voice_weather.run_command(username, text, context, db)
        #TV-Programm
        elif helper.contains_pattern(voice_patterns.PATTERN_TV, text):
            answer = Modules.voice_tv.run_command(username, text, context, db)
        elif helper.contains_pattern(voice_patterns.PATTERN_TV_TIPPS, text):
            answer = Modules.voice_tv_tipps.run_command(username, text, context, db)
        #Witz
        #elif contains_pattern(voice_patterns.PATTERN_JOKE, text):
        #    answer = get_joke(username, text, context, db)
        #Kalender
        elif helper.contains_pattern(voice_patterns.PATTERN_ADD_CALENDAR, text):
            answer = Modules.voice_add_calendar.run_command(username, text, context, db)
        elif helper.contains_pattern(voice_patterns.PATTERN_GET_CALENDAR, text):
            answer = Modules.voice_calendar.run_command(username, text, context, db)
        #Places
        elif helper.contains_pattern(voice_patterns.PATTERN_PLACES, text):
            answer = Modules.voice_places_api.get_places(username, text, context, db)
        #Summary
        elif helper.contains_pattern(voice_patterns.PATTERN_SUMMARY, text):
            answer = Modules.voice_summary.get_voice_summary(username, text, context, db)
        #Rezepte
        elif helper.contains_pattern(voice_patterns.PATTERN_RECIPES, text):
            answer = Modules.voice_recipes.run_command(username, text, context, db)
        #Wikipedia
        elif helper.contains_pattern(voice_patterns.PATTERN_WIKIPEDIA, text):
            answer = Modules.voice_wikipedia.run_command(username, text, context, db)
        #Movie Rating
        elif helper.contains_pattern(voice_patterns.PATTERN_MOVIE_RATING, text):
            answer = Modules.voice_movie_api.run_command(username, text, context, db)
        #Routendaten
        elif helper.contains_pattern(voice_patterns.PATTERN_ROUTE, text):
            answer = Modules.voice_route.get_distance_data(username, text, context, db)
        #Aktivitäten abfragen
        #elif contains_pattern(voice_patterns.PATTERN_ACTIVITY, text):
        #    answer = get_activities(username, text, context, db)
        #Sensordaten abfragen
        elif helper.contains_pattern(voice_patterns.PATTERN_SENSOR_DATA, text):
            answer = Modules.voice_get_sensor_data.run_command(username, text, context, db)
        elif helper.contains_pattern(voice_patterns.PATTERN_REED_SENSOR_DATA, text):
            answer = Modules.voice_get_sensor_data.get_reed_data(username, text, context, db)
        elif helper.contains_pattern(voice_patterns.PATTERN_PRESENCE_SENSOR_DATA, text):
            answer = Modules.voice_get_sensor_data.get_presence_data(username, text, context, db)
        #RGB Lichter steuern
        elif helper.contains_pattern(voice_patterns.PATTERN_RGB_CONTROL, text):
            answer = Modules.voice_rgb_control.run_command(username, text, context, db)
        #Nährwertmanager
        elif helper.contains_pattern(voice_patterns.PATTERN_QUERY_NUTRITION_DIARY, text):
            answer = Modules.voice_nutrition_diary.run_command(username, text, context, db)
        elif helper.contains_pattern(voice_patterns.PATTERN_NUTRITION_INFO, text):
            answer = Modules.voice_nutrition_manager.run_command(username, text, context, db)
        elif helper.contains_pattern(voice_patterns.PATTERN_ADD_NUTRITION_ITEM, text):
            answer = Modules.voice_add_nutrition_data.run_command(username, text, context, db)
        # Taschenrechner
        elif helper.contains_pattern(voice_patterns.PATTERN_CALCULATOR, text):
            answer = Modules.voice_calculator.calculator(username, text, context, db)
        #Unterhaltung
        else:
            answer = Modules.voice_conversation.conversation(username, text, context, db)

    return(answer)



#!/usr/bin/python
# -*- coding: utf-8 -*-
import socket
import traceback
from thread import start_new_thread

from constants import END_OF_MESSAGE
from db_utils import get_database_con

SPEAKER_HOST = ''
SPEAKER_PORT = 7888

def smart_speaker_loop():
    while(True):
        start_server_socket()
    return

def clientthread(conn):
    return


def listen_for_requests(s):
    # wait to accept a connection - blocking call
    conn, addr = s.accept()
    print('Connected with ' + addr[0] + ':' + str(addr[1]))

    # start new thread takes 1st argument as a function name to be run, second is the tuple of arguments to the function.
    start_new_thread(clientthread, (conn,))

def start_server_socket():
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    print('Socket created')

    # send_notification_to_users(['sascha'], "Titel", "Nachricht", get_database_con())

    # Bind socket to local host and port
    try:
        s.bind((SPEAKER_HOST, SPEAKER_PORT))
    except socket.error as msg:
        traceback.print_exc()
        print('Bind failed. Error Code : ' + str(msg[0]) + ' Message ' + msg[1])

    print('Socket bind complete')

    # Start listening on socket
    s.listen(10)
    print('Socket now listening')

    while 1:
        try:
            listen_for_requests(s)
        except:
            traceback.print_exc()
            break

    s.close()


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime
import json

from homevee.DeviceAPI.heating import get_thermostats
from homevee.Functions.calendar import get_calendar_day_items
from homevee.Functions.nutrition_data import get_user_nutrition_overview
from homevee.Functions.room_data import get_rooms, get_room_name
from homevee.Functions.sensor_data import get_sensor_data
from homevee.Functions.weather import get_weather
from homevee.Helper.helper_functions import has_permission
from homevee.device_types import *

STATIC_DASHBOARD_ITEMS = ['calendar', 'weather', 'tvprogramme', 'drivingtime', 'nutritionmanager']

def get_user_dashboard(username, db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM userdata WHERE USERNAME == :username",
                    {'username': username})

        data = cur.fetchone()

        user_dashboard = data['DASHBOARD_DATA']

        if user_dashboard is None:
            data = {'static': [], 'dynamic': []}
        else:
            data = json.loads(user_dashboard)

        dashboard_data = {}
        dashboard_data['static'] = data['static']

        if dashboard_data is not None and dashboard_data != "":

            dashboard_keys = []
            for item in dashboard_data['static']:
                print item
                dashboard_keys.append(item['id'])


            for item in STATIC_DASHBOARD_ITEMS:
                if item not in dashboard_keys:
                    print item+" not existing"
                    dashboard_data['static'].append({'id': item, 'checked': False})

            dashboard_data['dynamic'] = []

            '''
            dynamic_temp_items = data['dynamic']
            devices_map = {}
            dynamic_items = []

            print dynamic_temp_items

            for item in dynamic_temp_items:
                found = False

                if(item['type'] not in devices_map):
                    devices_map[item['type']] = []

                if item['id'] not in devices_map[item['type']]:
                    devices_map[item['type']].append(item['id'])
                else:
                    found = True

                if not found:
                    dynamic_items.append(item)

            print dynamic_items
            '''

            devices_map = {}
            for item in data['dynamic']:
                found = False

                if (item['type'] not in devices_map):
                    devices_map[item['type']] = []

                if item['id'] not in devices_map[item['type']]:
                    devices_map[item['type']].append(item['id'])
                else:
                    found = True

                if not found:
                    dashboard_data['dynamic'].append(get_device_info(username, item['type'], item['id'], db))

            return dashboard_data

    return {'result': 'unknownuser'}

def get_user_dashboard_items(username, db):
    dashboard_array = []

    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM USERDATA WHERE USERNAME == :username", {'username': username})

        db_data = cur.fetchone()

        if db_data is not None:
            data = db_data['DASHBOARD_DATA']

            if data is None:
                return {'dashboard': dashboard_array}

            data = json.loads(data)

            static_items = data['static']

            static_data = {}

            for item in static_items:
                static_data[item['id']] = item['checked']

            dynamic_temp_items = data['dynamic']
            devices_map = {}
            dynamic_items = []

            print dynamic_temp_items

            for item in dynamic_temp_items:
                found = False

                if(item['type'] not in devices_map):
                    devices_map[item['type']] = []

                if item['id'] not in devices_map[item['type']]:
                    devices_map[item['type']].append(item['id'])
                else:
                    found = True

                if not found:
                    dynamic_items.append(item)

            print dynamic_items

            # Wetter hinzufügen
            if 'weather' in static_data:
                if static_data['weather']:
                    weather_data = get_weather(1, db)
                    dashboard_array.append({'id': 'weather', 'data': weather_data[0]})

            # TV Programm hinzufügen
            if 'tvprogramme' in static_data:
                if static_data['tvprogramme']:
                    dashboard_array.append({'id': 'tvprogramme', 'data': None})

            #Kalender hinzufügen
            if 'calendar' in static_data:
                if static_data['calendar']:
                    calendar_entries = get_calendar_day_items(username, datetime.datetime.now().strftime("%Y-%m-%d"), db)

                    items = calendar_entries['calendar_entries']

                    data = None
                    for item in items:
                        start = datetime.datetime.strptime(item['start'], '%H:%M')
                        dnow = datetime.datetime.now()

                        if(dnow.time() > start.time()):
                            continue

                        if data is None:
                            data = item['name'] + " um " + item['start']
                        else:
                            data += '\n' + item['name'] + " um " + item['start']+ " Uhr"

                    if data is None:
                        data = 'Keine Termine heute'

                    dashboard_array.append({'id': 'calendar', 'data': data})

            # Fahrzeit
            if 'drivingtime' in static_data:
                if static_data['drivingtime']:
                    dashboard_array.append({'id': 'drivingtime', 'data': '27 min'})

            # Ernährungsmanager
            if 'nutritionmanager' in static_data:
                if static_data['nutritionmanager']:
                    calories_left = get_user_nutrition_overview(username, db)['data']['nutrition_day_data']['calories_left']

                    if(calories_left >= 0):
                        text = str(calories_left)+' Kalorien übrig'
                    else:
                        text = str(calories_left)+' Kalorien über deinem Ziel'

                    dashboard_array.append({'id': 'nutritionmanager', 'data': text})

            if len(dynamic_items) > 0:
                dashboard_array.append({'id': 'favouritedevices', 'data': None})

        return {'dashboard': dashboard_array}

def edit_user_dashboard(username, dashboard_data, db):
    with db:
        cur = db.cursor()

        dashboard_data = json.loads(dashboard_data)

        #Check duplicates

        dynamic_temp_items = dashboard_data['dynamic']
        devices_map = {}
        dynamic_items = []

        print dynamic_temp_items

        for item in dynamic_temp_items:
            found = False

            if (item['type'] not in devices_map):
                devices_map[item['type']] = []

            if item['id'] not in devices_map[item['type']]:
                devices_map[item['type']].append(item['id'])
            else:
                found = True

            if not found:
                dynamic_items.append(item)

        print dynamic_items

        dashboard_data['dynamic'] = dynamic_items

        #Check duplicates end

        dashboard_data_json = json.dumps(dashboard_data)

        cur.execute("UPDATE USERDATA SET DASHBOARD_DATA = :data WHERE USERNAME == :username",
                    {'data': dashboard_data_json, 'username': username})

        cur.close()

        return get_user_dashboard(username, db)

def get_user_favourite_devices(username, db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM USERDATA WHERE USERNAME == :username", {'username': username})

        data = json.loads(cur.fetchone()['DASHBOARD_DATA'])

        dynamic_items = data['dynamic']

        print "data: "+str(data)

        fav_array = get_dynamic_items(username, dynamic_items, db)

        return fav_array

def get_dynamic_items(username, dynamic_items, db):
    rooms = get_rooms(username, db)['rooms']

    favourites = {}

    room_map = {}

    for room in rooms:
        favourites[int(room['location'])] = []
        room_map[room['location']] = room

    types = {}

    for item in dynamic_items:
        #print item
        if item['type'] not in types:
            types[item['type']] = []

        types[item['type']].append(item['id'])
        print types

    for type in types:
        ids = types[type]
        if len(ids) > 0:
            for id in ids:
                item = get_device_info(username, type, id, db)

                if item is None:
                    continue

                print favourites

                favourites[int(item['room'])].append(item)

    output = []
    for location in favourites.keys():
        room_item = room_map[location]

        if len(favourites[location]) > 0:
            room_item['devices'] = favourites[location]
            output.append(room_item)

    return output

def get_device_info(username, type, id, db):

    print type + " - " + id

    with db:
        cur = db.cursor()

        device_data = None

        try:
            if type == FUNKSTECKDOSE:
                cur.execute("SELECT * FROM FUNKSTECKDOSEN WHERE DEVICE == :id", {'id': id})
                item = cur.fetchone()
                device_data = {'room': item['ROOM'], 'name': item['NAME'], 'icon': item['ICON'],
                               'value': item['ZUSTAND'] == "1",
                               'type': 'switch', 'devicetype': type, 'id': id}
            elif type == PHILIPS_HUE_LIGHT:
                cur.execute("SELECT * FROM PHILIPS_HUE_LIGHTS WHERE ID == :id", {'id': id})
                item = cur.fetchone()
                device_data = {'room': item['LOCATION'], 'name': item['NAME'],
                               'icon': item['ICON'],
                               'value': {'is_on': False, 'brightness': 100, 'color': item['HUE']},
                               'type': 'rgb', 'devicetype': type, 'id': id}
            #URL-Geräte
            elif type == URL_RGB_LIGHT:
                cur.execute("SELECT * FROM URL_RGB_LIGHT WHERE ID = :id", {'id': id})
                item = cur.fetchone()
                device_data = {'name': item['NAME'], 'room': item['LOCATION'], 'id': item['ID'], 'type': 'rgb',
                               'icon': item['ICON'], 'value': {'is_on': False, 'brightness': 100, 'color': item['LAST_COLOR']}}
            elif type == URL_TOGGLE:
                cur.execute("SELECT * FROM URL_TOGGLE WHERE ID = :id",{'id': id})
                item = cur.fetchone()
                device_data = {'name': item['NAME'], 'room': item['LOCATION'], 'id': item['ID'],
                                    'icon': item['ICON'], 'value': '', 'type': 'toggle'}
            #Z-Wave
            elif type == ZWAVE_THERMOSTAT:
                cur.execute("SELECT * FROM ZWAVE_THERMOSTATS WHERE THERMOSTAT_ID == :id", {'id': id})
                item = cur.fetchone()
                data = get_thermostats(username, item['RAUM'], type, item['THERMOSTAT_ID'], db)
                device_data = {'name': item['NAME'], 'id': item['THERMOSTAT_ID'],
                        'icon': item['ICON'], 'room': item['RAUM'], 'type': 'heating',
                        'value': data['value'], 'min': data['min'], 'max': data['max'], 'step': 0.5}
            elif type == ZWAVE_DIMMER:
                cur.execute("SELECT * FROM ZWAVE_DIMMER WHERE ID == :id", {'id': id})
                item = cur.fetchone()
                device_data = {'name': item['NAME'], 'id': item['THERMOSTAT_ID'],
                        'icon': item['ICON'], 'room': item['LOCATION'], 'type': 'heating',
                        'value': item['VALUE']}
            elif type == ZWAVE_POWER_METER:
                cur.execute("SELECT * FROM ZWAVE_THERMOSTATS WHERE DEVICE_ID == :id", {'id': id})
                item = cur.fetchone()
                device_data = {'name': item['NAME'], 'id': item['DEVICE_ID'],
                        'icon': item['ICON'], 'room': item['LOCATION'], 'type': 'heating',
                        'value': item['VALUE']}
            elif type == ZWAVE_SENSOR:
                cur.execute("SELECT * FROM ZWAVE_SENSOREN WHERE ID == :id", {'id': id})
                item = cur.fetchone()
                device_data = {'name': item['SHORTFORM'], 'id': item['ID'],
                        'icon': item['ICON'], 'room': item['RAUM'], 'type': 'value',
                               'value': get_sensor_data(username, item['RAUM'], type, id, 1, db)}
            elif type == ZWAVE_SWITCH:
                cur.execute("SELECT * FROM ZWAVE_SWITCHES WHERE THERMOSTAT_ID == :id", {'id': id})
                item = cur.fetchone()
                device_data = {'name': item['NAME'], 'id': item['THERMOSTAT_ID'],
                        'icon': item['ICON'], 'room': item['LOCATION'], 'type': 'heating',
                        'value': item['VALUE']}
            #MAX Cube
            elif type == MAX_THERMOSTAT:
                cur.execute("SELECT * FROM MAX_THERMOSTATS WHERE ID == :id", {'id': id})
                item = cur.fetchone()
                data = get_thermostats(username, item['RAUM'], type, id, db)
                device_data = {'name': item['NAME'], 'id': item['ID'], 'room': item['RAUM'], 'type': 'heating',
                        'icon': item['ICON'],
                        'value': data['value'], 'min': data['min'], 'max': data['max'], 'step': 0.5}
            #Wake on Lan
            elif type == WAKE_ON_LAN:
                cur.execute("SELECT * FROM WAKE_ON_LAN WHERE DEVICE == :id", {'id': id})
                item = cur.fetchone()
                device_data = {'name': item['NAME'], 'id': item['DEVICE'], 'icon': item['ICON'], 'room': item['LOCATION'],
                               'value': '', 'type': 'wakeonlan'}
            elif type == XBOX_ONE_WOL:
                cur.execute("SELECT * FROM XBOX_ONE_WOL WHERE ID == :id", {'id': id})
                item = cur.fetchone()
                device_data = {'name': item['NAME'], 'id': item['ID'], 'icon': item['ICON'], 'room': item['LOCATION'],
                               'value': '', 'type': 'wakeonlan'}

            #MQTT Sensor
            elif type == MQTT_SENSOR:
                cur.execute("SELECT * FROM MQTT_SENSORS WHERE ID == :id", {'id': id})
                item = cur.fetchone()
                device_data = {'name': item['NAME'], 'id': item['ID'],
                               'icon': item['ICON'], 'room': item['ROOM'], 'type': 'value',
                               'value': get_sensor_data(username, item['ROOM'], type, id, 1, db)}
        except:
            print "Error loading device"

        if device_data is None or not has_permission(username, device_data['room'], db):
            return None

        device_data['roomname'] = get_room_name(device_data['room'], db)
        device_data['devicetype'] = type

        return device_data

    '''//DIY
		case "DIY Gerät":
			$dbQuery = $db->prepare("SELECT * FROM 'DIY_DEVICES' WHERE DEVICE == :id");
			$dbQuery->execute($queryParams);
			
			
			$query = $dbQuery->fetch(PDO::FETCH_ASSOC);
			$devicedata = array("room" => $query['ROOM'], "name" => $query['NAME'], "icon" => $query['ICON'],
			"value" => "", "type" => "button");
			break;
		case "DIY Sensor":
			$dbQuery = $db->prepare("SELECT * FROM 'DIY_SENSORS' WHERE ID == :id");
			$dbQuery->execute($queryParams);
			
			
			$query = $dbQuery->fetch(PDO::FETCH_ASSOC);
			$devicedata = array("room" => $query['RAUM'], "name" => $query['NAME'], "icon" => $query['ICON'],
			"value" => getSensorData($query['RAUM'], $type, $id, "1", $db), "type" => "value");
		case "DIY Fensterkontakt":
			$dbQuery = $db->prepare("SELECT * FROM 'DIY_REEDSENSORS' WHERE ID == :id");
			$dbQuery->execute($queryParams);
			
			
			$query = $dbQuery->fetch(PDO::FETCH_ASSOC);
			$devicedata = array("room" => $query['RAUM'], "name" => $query['NAME'], "icon" => $query['ICON'],
			"value" => getReedData($query['RAUM'], $type, $id, $db), "type" => "reed");
			break;
		case "DIY Anwesenheitssensor":
			$dbQuery = $db->prepare("SELECT * FROM 'DIY_PRESENCESENSORS' WHERE ID == :id");
			$dbQuery->execute($queryParams);
			
			
			$query = $dbQuery->fetch(PDO::FETCH_ASSOC);
			$devicedata = array("room" => $query['RAUM'], "name" => $query['NAME'], "icon" => $query['ICON'],
			"value" => getReedData($query['RAUM'], $type, $id, $db), "type" => "presence");
			break;
		case "DIY Schalter":
			$dbQuery = $db->prepare("SELECT * FROM 'DIY_SWITCHES' WHERE ID == :id");
			$dbQuery->execute($queryParams);
			
			
			$query = $dbQuery->fetch(PDO::FETCH_ASSOC);
			$devicedata = array("room" => $query['RAUM'], "name" => $query['NAME'], "icon" => $query['ICON'],
			"value" => getModes($query['RAUM'], $type, $id, $db) === "1", "type" => "switch");
			break;
		case "IP Kamera":
			$dbQuery = $db->prepare("SELECT * FROM 'IP_CAMERA' WHERE ID == :id");
			$dbQuery->execute($queryParams);
			
			$query = $dbQuery->fetch(PDO::FETCH_ASSOC);
			
			$ip = $query['IP'];
			$port = $query['PORT'];
				
			if($port != null && $port != ""){
				$port = ":$port";
			}
				
			$path = $query['PATH'];
				
			$url = "http://$ip$port$path";
			
			$devicedata = array('room' => $query['LOCATION'], 'name' => $query['NAME'], 'icon' => $query['ICON'],
			'value' => "", 'url' => $url, 'username' => $query['USERNAME'], 'password' => $query['PASSWORD'],
			'type' => "ipcamera");
			break;
			
		default:
			return null;
	}
	
	if(!hasPermission($devicedata['room'], $db) || $devicedata['room'] == null){
		return null;
	}
	else{
		$devicedata['roomname'] = $roomArray[$devicedata['room']];
		$devicedata['devicetype'] = $type;
		$devicedata['id'] = $id;
		
		return $devicedata;
	}
}
?>
    '''



#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import requests
from homevee.Manager.gateway import get_gateway


def do_zwave_request(path, db):
    gateway_data = get_gateway("Z-Wave", db)
    port = ""
    if (gateway_data['PORT'] != None and gateway_data['PORT'] != ""):
        port = ":" + str(gateway_data['PORT'])

    url = "http://" + gateway_data['KEY1'] + ':' + gateway_data['KEY2'] + '@' + gateway_data[
        'IP'] + port + path

    #print url

    headers = {'content-type': 'application/json', 'Accept-Charset': 'UTF-8'}
    request = requests.get(url, headers=headers)
    data = request.content

    #print data

    json_data = json.loads(data)

    #print url, json_data

    return json_data


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime
from time import sleep

from homevee.Functions.calendar import get_calendar_day_items
from homevee.Functions.weather import get_weather
from homevee.db_utils import get_database_con
from homevee.firebase_utils import send_notification_to_users


def send_overview_notifications():
    db = get_database_con()

    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM USERDATA")

        users = cur.fetchall()

        for user in users:
            day_overview = ""

            weather = get_weather(1, db)
            weather_text = "Wetter: " + weather[0]['desc'] + ", " + str(weather[0]['temps']['min']) + "°C - " + str(weather[0]['temps']['max']) + "°C"

            today = datetime.datetime.today()
            date_today = today.strftime("%Y-%m-%d")
            calendar = get_calendar_day_items(user['USERNAME'], date_today, db)
            calendar_items = calendar['calendar_entries']
            if len(calendar_items) == 0:
                calendar_text = "Keine Termine"
            else:
                if len(calendar_items) > 1:
                    calendar_text = str(len(calendar_items)) + " Termine: "
                    for i in range(0, len(calendar_items)):
                        if i == 0:
                            calendar_text += calendar_items[i]['name'] + " um " + calendar_items[i]['start'] + " Uhr"
                        else:
                            calendar_text += ", " + calendar_items[i]['name'] + " um " + calendar_items[i]['start'] + " Uhr"
                else:
                    calendar_text = "Ein Termin: "+calendar_items[0]['name']+" um "+calendar_items[0]['start']+" Uhr"

            day_overview = weather_text+"\n"+calendar_text

            send_notification_to_users([user['USERNAME']], "Deine Tagesübersicht", day_overview, db)


def init_thread():
    while(True):
        #wait until midnight + 5 minutes
        t = datetime.datetime.today()

        seconds_to_wait = (24-t.hour)*60*60 - ((t.minute * 60) - t.second) + 5*60 #Warten bis 00:05
        #print "warten für benachrichtigung: " + str(seconds_to_wait) + " sekunden"


        #print "waiting until: "+ str(datetime.datetime.now() + datetime.timedelta(seconds=seconds_to_wait))

        #print "waiting for: "+str(seconds_to_wait)

        sleep(seconds_to_wait)

        send_overview_notifications()


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import urllib

from homevee.Manager.gateway import get_gateway
from homevee.gateway_keys import RADEMACHER_HOMEPILOT

def control_blinds(id, goto, db):
    data = get_gateway(RADEMACHER_HOMEPILOT, db)
    ip = data['IP']

    url = "http://"+ip+"/deviceajax.do?cid=9&did="+str(id)+"&goto="+str(goto)+"&command=0"

    print url

    response = urllib.urlopen(url).read()

    data = json.loads(response)

    if(data['status'] != 'uisuccess'):
        return {'result': 'error'}

    print response

    with db:
        cur = db.cursor()

        cur.execute("UPDATE HOMEPILOT_BLIND_CONTROL SET LAST_POS = :pos WHERE ID == :id",
                    {'pos': goto, 'id': id})
    return {'result': 'ok'}


#!/usr/bin/python
# -*- coding: utf-8 -*-
import random

import homevee.Functions
from homevee.VoiceAssistant.helper import generate_string

def add_to_shopping_list(username, text, context, db):
    '''words = text.split(" ")

    for i in range(0, len(words)):
        word = words[i]



    if item_count > 1:
        answer_data = [
            ['Ok']
        ]
    else:
        answer_data = [
            ['Ok']
        ]

    output = generate_string(answer_data)'''

    output = "Add Shopping List"

    return {'msg_speech': output, 'msg_text': output}

def get_shopping_list(username, text, context, db):
    items = find_items(text, db)

    if len(items) is 0:
        #Ganze Liste abfragen
        items = Functions.shopping_list.get_shopping_list(username, db)['items']

    data = [
        [['Das ', 'Folgendes '], 'steht ', 'auf ', [[['der ', 'deiner '], 'Einkaufsliste'], [['dem ', 'deinem '], 'Einkaufszettel']], ': '],
        [['Diese ', 'Folgende '], ['Artikel ', 'Produkte '], ['stehen ', 'sind '], 'auf ', [[['der ', 'deiner '], 'Einkaufsliste'],
                                                                                          [['dem ', 'deinem '], 'Einkaufszettel']], ': ']
    ]

    output = generate_string(data)

    for i in range(0, len(items)):
        item = items[i]

        if len(items) > 1:
            #Mehr als ein Element
            if i is len(items) - 1:
                # Letztes Element
                output = output + " und "
            elif i < len(items) - 1:
                # Nicht erstes und nicht letztes Element
                output = output + ", "

        output = output + str(item['amount']) + " mal " + item['item']

    return {'msg_speech': output, 'msg_text': output}

def find_items(text, db):
    items = []

    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM SHOPPING_LIST")

        for item in cur.fetchall():
            position = text.find(item['ITEM'].lower())
            if position is not -1:
                items.append({'item': item['ITEM'], 'amount': item['AMOUNT'], 'id': item['ID']})

    return items


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="Homevee",
    version="0.1.1.1",
    author="Homevee",
    author_email="support@homevee.de",
    description="Dein neues Smarthome-System!",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://bitbucket.org/homevee/homevee",
    #scripts=['bin/homevee-server'],
    entry_points = {
        'console_scripts': ['homevee=homevee.command_line:main'],
    },
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'python-dateutil',
        'pyOpenSSL',
        'tensorflow',
        'tensorflow-gpu',
        'pymax',
        'paho-mqtt'
    ]
)



#!/usr/bin/python
# -*- coding: utf-8 -*-
import urllib2

from homevee.Manager.api_key import get_api_key
from homevee.VoiceAssistant.voice_patterns import PATTERN_MOVIE_RATING


def get_pattern(db):
    return PATTERN_MOVIE_RATING

def get_label():
    return "movierating"

def run_command(username, text, context, db):
    return get_movie_rating(username, text, context, db)

def get_movie_rating(username, text, context, db):
    api_key = get_api_key('TMDB', db)

    words = text.split(" ")

    query = None

    is_query = False

    film_index = -1

    for i in range(0, len(words)):
        word = words[i]
        if is_query:
            if query is None:
                query = word.capitalize()
            else:
                query = query + " " + word.capitalize()

        if word == "film":
            if not is_query:
                film_index = i
            is_query = True

        if is_query and i > film_index and query != None:
            if word == "gut" or word == "schlecht":
                break

    #url = 'https://api.themoviedb.org/3/movie/'+movie_id+'?api_key='+api_key+'&language=de/'
    #response = urllib2.urlopen(url).read()
    #print(response)

    #return {'msg_speech': output, 'msg_text': output}


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import socket
import ssl
import traceback

import time
from thread import start_new_thread
from urllib import urlencode
from urllib2 import Request, urlopen

from Helper import compression
from Helper.helper_functions import remote_control_enabled
from API import process_data
from constants import END_OF_MESSAGE
from db_utils import get_database_con, set_server_data, get_server_data

CLOUD_URL = "free.cloud.homevee.de"
CLOUD_PORT = 7778

#Verbindung zu Cloud herstellen

#Auf Befehl warten, ausführen und Ergebnis zurück an Server senden

def cloud_connection_loop():
    while(True):
        connect_to_cloud()

def keep_connection_alive(conn, time):
    while True:
        try:
            #Send some data every minute
            time.sleep(time)
            print "sending ping"
            conn.send(1)
        except:
            break

def connect_to_cloud():
    db = get_database_con()

    conn = None

    if(not remote_control_enabled(db)):
        print "REMOTE CONTROL NOT ENABLED"
        #time.sleep(5 * 60)  # 5 Minuten warten
        #return

    try:
        conn = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        #conn.settimeout(5*60)

        #conn.connect((CLOUD_URL, CLOUD_PORT))

        #print("Connected")

        '''
        conn = ssl.wrap_socket(conn,
                                     ca_certs=SSL_FULLCHAIN,
                                     cert_reqs=ssl.CERT_REQUIRED,
                                     ssl_version=ssl.PROTOCOL_SSLv23)
        '''

        #SSL aktivieren
        conn = ssl.wrap_socket(conn)

        #print("Wrapping SSL")

        print("Connecting to cloud ("+CLOUD_URL+")...")
        conn.connect((CLOUD_URL, CLOUD_PORT))

        start_new_thread(keep_connection_alive, (conn, 60))

        #print ("SSL connected")

        #print("Connection successful!")

        REMOTE_ID, ACCESS_TOKEN = get_remote_data(db)

        if REMOTE_ID is None or ACCESS_TOKEN is None:
            REMOTE_ID, ACCESS_TOKEN = register_to_cloud(db)
            print "NO CLOUD CONNECTION CREDENTIALS"

        data_dict = {'access_token': ACCESS_TOKEN, 'remote_id': REMOTE_ID}

        data = json.dumps(data_dict)+END_OF_MESSAGE

        len_send = conn.send(data)

        while 1:
            data = conn.recv(8192)

            if data == "":
                db.close()
                conn.close()
                return

            print("")
            print("Received from Cloud: " + data)

            data_parts = data.split(END_OF_MESSAGE)
            for data_part in data_parts:
                if data_part is None or data_part == "":
                    continue

                print("")
                print(data_part)

                data_dict = json.loads(data_part)

                if 'result' in data_dict:
                    print(data_dict)

                    if(data_dict['result'] == "error"):
                        print "couldn't connect to cloud"
                        time.sleep(60) #1 Minute warten

                else:
                    try:
                        client_id = data_dict['client_id']

                        data_dict = json.loads(data_dict['msg'])

                        msg = process_data(data_dict, db)

                        #start_time = time.time()
                        #compressed_message = compression.compress_string(msg)
                        #end_time = time.time()

                        #print "uncompressed: " + str(len(msg))
                        #print "compressed: " + str(len(compressed_message)) + ", time: " + str(end_time - start_time)
                        #compressed_message = compressed_message.decode('iso-8859-1').encode('utf8')

                        data = {}
                        data['msg'] = msg
                        data['client_id'] = client_id
                        data = json.dumps(data)

                        if data is None:
                            data = {'status': 'error'}
                            data['client_id'] = client_id
                            data = json.dumps(data)
                        #else:
                        #    data['client_id'] = client_id
                        #    data['msg'] = json.dumps(msg)
                        #    data = json.dumps(data)

                        len_send = conn.send(data+END_OF_MESSAGE)

                        print("Sent to remote Server ("+data_dict['action']+")(client: "+str(client_id)+"): "+data)
                    except Exception, e:
                        #traceback.print_exc()
                        if 'status' in data_dict:
                            if data_dict['status'] == 'connectionok':
                                print('connectionok')
                            if data_dict['status'] == 'nocredentials':
                                #Credentials wrong
                                #deleting them from db
                                with db:
                                    print "wrongcredentials"
                                    time.sleep(60) #1 Minute warten
                                    #cur = db.cursor()
                                    #cur.execute("DELETE FROM SERVER_DATA WHERE KEY IN ('REMOTE_ID', 'REMOTE_ACCESS_TOKEN')")

    except Exception as e:
        traceback.print_exc()

        if conn is not None:
            conn.close()

    db.close()

def save_remote_control_data(remote_id, access_token, db):
    with db:
        cur = db.cursor()

        #cur.execute("INSERT INTO SERVER_DATA (KEY, VALUE) VALUES (:key, :value)",
        #            {'key': 'REMOTE_ID', 'value': remote_id})
        #cur.execute("INSERT INTO SERVER_DATA (KEY, VALUE) VALUES (:key, :value)",
        #            {'key': 'REMOTE_ACCESS_TOKEN', 'value': access_token})

        REMOTE_ID, REMOTE_SECRET = get_remote_data(db)

        if REMOTE_ID is None or REMOTE_SECRET is None:
            print("insert remote data")
            cur.execute("INSERT INTO SERVER_DATA (KEY, VALUE) VALUES ('REMOTE_ID', :remote_id)",
                    {'remote_id': remote_id})
            cur.execute("INSERT INTO SERVER_DATA (KEY, VALUE) VALUES ('REMOTE_ACCESS_TOKEN', :token)",
                    {'token': access_token})
        else:
            print("update remote data")
            cur.execute("UPDATE SERVER_DATA SET VALUE = :remote_id WHERE KEY = 'REMOTE_ID'",
                {'remote_id': remote_id})

            cur.execute("UPDATE SERVER_DATA SET VALUE = :token WHERE KEY = 'REMOTE_ACCESS_TOKEN'",
                {'token': access_token})

#Returns a tuple: (REMOTE_ID, PRIVATE_KEY)
def get_remote_data(db):
    with db:
        cur = db.cursor()

        #cur.execute("DELETE FROM SERVER_DATA WHERE KEY IN ('REMOTE_ACCESS_TOKEN', 'REMOTE_ID')")

        cur.execute("SELECT VALUE FROM SERVER_DATA WHERE KEY = 'REMOTE_ID'")

        result = cur.fetchone()

        if(result is None):
            return None, None
        else:
            remote_id = result['VALUE']
            print("Remote-ID: "+remote_id)

        cur.execute("SELECT VALUE FROM SERVER_DATA WHERE KEY = 'REMOTE_ACCESS_TOKEN'")

        result = cur.fetchone()

        if(result is None):
            return None, None
        else:
            remote_access_token = result['VALUE']
            print("Access Token: "+remote_access_token)

    return (remote_id, remote_access_token)

def register_to_cloud(db):
    url = 'https://cloud.homevee.de/server-api.php'  # Set destination URL here
    post_fields = {'action': 'generateremoteid'}  # Set POST fields here

    request = Request(url, urlencode(post_fields).encode())
    response = urlopen(request).read().decode()

    data = json.loads(response)

    remote_id = data['remote_id']
    access_token = data['access_token']

    #save remote id
    set_server_data("REMOTE_ID", remote_id, db)
    set_server_data("REMOTE_ACCESS_TOKEN", access_token, db)

    return remote_id, access_token

def register_to_cloud_2(db):
    try:
        print("Registering at cloud...")
        conn = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        #conn.connect((CLOUD_URL, CLOUD_PORT))

        #print("Connected")
        '''
        conn = ssl.wrap_socket(conn,
                                   ca_certs=SSL_FULLCHAIN,
                                   cert_reqs=ssl.CERT_REQUIRED,
                                   ssl_version=ssl.PROTOCOL_SSLv23)
        '''
        #print("Wrapping SSL")

        # SSL aktivieren
        conn = ssl.wrap_socket(conn)

        conn.connect((CLOUD_URL, CLOUD_PORT))

        #print ("SSL connected")

        len_send = conn.send('registration'+END_OF_MESSAGE)

        #print("Sent data")

        while 1:
            data = conn.recv(8192)

            if data == "":
                db.close()
                conn.close()
                return
            print("Received: " + data)

            data_dict = json.loads(data)

            if 'result' in data_dict:
                if data_dict['result'] == "registrationok":
                    remote_id = data_dict['remote_id']
                    access_token = data_dict['access_token']

                    save_remote_control_data(remote_id, access_token, db)

                    print("Registration successful!")

                    return remote_id, access_token

    except Exception, e:
        traceback.print_exc()


#!/usr/bin/python
# -*- coding: utf-8 -*-

SET_MODES = "setmodes"
GET_SENSOR_DATA = "getsensordata"
SET_HEATING = "setheating"
SET_THERMOSTAT = "setthermostat"
CONTROL_BLINDS = "controlblinds"
GET_WEATHER = "getweather"
ADD_NUTRITION = "addnutrition"
TV_PROGRAMME = "tvprogramme"
WOL = "wol"


#!/usr/bin/python
# -*- coding: utf-8 -*-
import os
import traceback
import urllib
import urllib2

import VoiceAssistant
from constants import HOMEVEE_DIR
from db_utils import get_database_con

CURRENT_VERSION = "0.0";

def check_for_updates():
    try:
        url = VoiceAssistant.SMART_API_PATH + "/?action=check_for_updates&current_version=" + urllib.quote(CURRENT_VERSION.encode('utf8'))
        print url
        data = urllib2.urlopen(url).read()
    except Exception, e:
        traceback.print_exc()
        data = None

def update_system():
    update_database()
    update_files()

def update_database():

    commands = []

    db = get_database_con()

    with db:
        cur = db.cursor()

        for command in commands:
            cur.execute(command)

    return

def update_files():
    #os.system("sudo rm -r "+HOMEVEE_DIR)

    #Neue Homevee-Version mit "git clone" herunterladen
    #os.chdir(os.path.dirname(HOMEVEE_DIR))

    #HOMEVEE_REPOSITORY = 'https://bitbucket.org/smarthomeblogger/shacos.git'

    #os.system("sudo git clone " + HOMEVEE_REPOSITORY)

    #os.system("sudo reboot")
	
	return


#!/usr/bin/python
# -*- coding: utf-8 -*-
from homevee.DeviceAPI import philips_hue, blind_control
from homevee.DeviceAPI.heating import heating_control
from homevee.DeviceAPI.set_modes import set_modes
from homevee.VoiceAssistant.context_keys import *
from homevee.VoiceAssistant.helper import generate_string, set_context, find_room, get_okay
from homevee.VoiceAssistant.voice_patterns import GET_PATTERN_SET_MODES
from homevee.device_types import *

def get_pattern(db):
    return GET_PATTERN_SET_MODES(db)

def get_label():
    return "switch"

def run_command(username, text, context, db):
    return set_modes_voice(username, text, context, db)

def set_modes_voice(username, text, context, db):
    room = find_room(text, db)

    if room is not None:
        room_key = room['LOCATION']
    else:
        room_key = None

    devices = find_devices(text, room_key, db)

    if len(devices) == 0:
        #Keine Geräte gefunden
        answer_data = [
            [['Dieses ', 'Das genannte '], 'Gerät ', ['existiert nicht.', 'gibt es nicht.', 'wurde noch nicht angelegt.']]
        ]

        answer = generate_string(answer_data)

        return {'msg_speech': answer, 'msg_text': answer}

    words = text.split(" ")
    mode = None
    on_words = ['an', 'ein']
    off_words = ['aus']
    for word in words:
        if word in on_words:
            mode = 1
            break
        elif word in off_words:
            mode = 0
            break

    if mode is None:
        if 'mode' in context:
            mode = context['mode']
        else:
            # Kein Zustand angegeben
            answer_data = [
                ['Soll ', 'ich ', 'die Geräte ', ['an', 'ein'], '- oder aus', ['schalten', 'machen'], [', '+username, ''], '?']
            ]

            answer = generate_string(answer_data)

            return {'msg_speech': answer, 'msg_text': answer}

    #Geräte schalten
    for device in devices:
        if(device['type'] == PHILIPS_HUE_LIGHT):
            philips_hue.set_light_mode(username, device['id'], mode, None, None, None, db)
        else:
            set_modes(username, device['type'], device['id'], mode, db)

    set_context(username, SET_MODES, {'location': room, 'devices': devices, 'mode':mode}, db)

    MODE_WORDS = on_words
    if mode == 0:
        MODE_WORDS = off_words

    DEVICE_STRING = None
    if len(devices) > 1:
        VERB = "wurden"
        DEVICE_WORD = "Die Geräte "

        for i in range(0, len(devices)):
            if DEVICE_STRING is None:
                DEVICE_STRING = '' + devices[i]['name'] + ''
            elif i == len(devices) - 1:
                DEVICE_STRING = DEVICE_STRING + ' und ' + '' + devices[i]['name'] + ''
            else:
                DEVICE_STRING = DEVICE_STRING + ', ' + '' + devices[i]['name'] + ''

    else:
        DEVICE_WORD = "Das Gerät "
        VERB = "wurde"

        DEVICE_STRING = '' + devices[0]['name'] + ''

    answer_data = [
        [get_okay(), [', ' + username, ''], '.',
         ['', [' ', DEVICE_WORD,  [DEVICE_STRING + ' ', ''], VERB + ' ',
               MODE_WORDS, ['geschaltet', 'geschalten'], '.']]]
    ]

    answer = generate_string(answer_data)

    return {'msg_speech': answer, 'msg_text': answer}

def control_blinds(username, text, context, db):
    words = text.split(" ")

    value = None

    if context is not None:
        if 'value' in context:
            value = int(context['value'])

    if value is None:
        if 'auf' not in words or len(words) <= words.index('auf') + 1:
            close_words = ['zu', 'runter', 'herunter']
            open_words = ['auf', 'rauf', 'hoch']

            for word in words:
                if word in close_words:
                    value = 100
                    break
                elif word in open_words:
                    value = 0
                    break
        else:
            value = words[words.index('auf') + 1]

    room = find_room(text, db)
    if room is not None:
        room_key = room['LOCATION']
    else:
        if context is not None and 'room' in context and context['room'] is not None:
            room = context['room']
            room_key = context['room']['LOCATION']
        else:
            # no room => control in every room
            room = None
            '''
            answer_data = [
                [['Wo ', 'In welchem Raum '], 'soll ich das machen', [', ' + username, ''], '?']
            ]

            if value is not None:
                set_context(username, CONTROL_BLINDS, {'value': value}, db)

            answer = generate_string(answer_data)

            return {'msg_speech': answer, 'msg_text': answer}
            '''

    print str(room) + ": " + str(value)

    with db:
        cur = db.cursor()

        blinds = []

        blinds_db_data = [
            {'table': 'HOMEPILOT_BLIND_CONTROL', 'room_col': 'LOCATION', 'id_col': 'ID', 'type': RADEMACHER_BLIND_CONTROL}
        ]

        for data_item in blinds_db_data:
            if room is not None:
                cur.execute('SELECT * FROM ' + data_item['table'] + ' WHERE ' + data_item['room_col'] + ' = :room',
                        {'room': room_key})
            else:
                cur.execute('SELECT * FROM ' + data_item['table'])

            for item in cur.fetchall():
                blinds_item = {'type': data_item['type'], 'id': item[data_item['id_col']]}
                blinds.append(blinds_item)

        print blinds

        for blinds_item in blinds:
            blind_control.set_blinds(username, blinds_item['type'], blinds_item['id'], value, db)

    # set_context(username, SET_THERMOSTAT, {'room': room, 'value': value}, db)

    bezeichner = ['Der Rollladen wurde', 'Die Rolläden wurden', 'Die Jalousien wurden']

    if room is None:
        bezeichner = ['Alle Rolläden wurden', 'Alle Jalousien wurden', 'Jeder Rollladen wurde']

    opened_words = ['geöffnet', 'hochgefahren']
    closed_words = ['geschlossen', 'runtergefahren']

    value_string = ['auf '+str(value)+' % ', ['gefahren', 'eingestellt']]

    if value is 0:
        action_string = opened_words
    elif value is 100:
        action_string = closed_words
    else:
        action_string = value_string

    answer_data = [
        [get_okay(), [', ' + username, ''], '.',
         ['', [' ', bezeichner, ' ', action_string, '.']]]
    ]

    answer = generate_string(answer_data)

    return {'msg_speech': answer, 'msg_text': answer}

def set_thermostat(username, text, context, db):
    words = text.split(" ")

    if 'auf' not in words or len(words) < words.index('auf')+1:
        if context is not None and 'value' in context and context['value'] is not None:
            value = context['value']
        else:
            answer_data = [
                [['Welchen Wert ', 'Welche Temperatur'], ' soll ich einstellen', [', ' + username, ''], '?']
            ]

            set_context(username, SET_THERMOSTAT, context, db)

            answer = generate_string(answer_data)

            return {'msg_speech': answer, 'msg_text': answer}
    else:
        value = words[words.index('auf')+1]

    room = find_room(text, db)
    if room is not None:
        room_key = room['LOCATION']
    else:
        if context is not None and 'room' in context and context['room'] is not None:
            room = context['room']
            room_key = context['room']['LOCATION']
        else:
            # no room
            answer_data = [
                [['Wo ', 'In welchem Raum '], 'soll ich das machen', [', ' + username, ''], '?']
            ]

            if value is not None:
                set_context(username, SET_THERMOSTAT, {'value': value}, db)

            answer = generate_string(answer_data)

            return {'msg_speech': answer, 'msg_text': answer}

    print str(room)+": "+value

    with db:
        cur = db.cursor()

        thermostats = []

        thermostat_db_data = [
            {'table': 'MAX_THERMOSTATS', 'room_col': 'RAUM', 'id_col': 'ID', 'type': MAX_THERMOSTAT},
            {'table': 'ZWAVE_THERMOSTATS', 'room_col': 'RAUM', 'id_col': 'THERMOSTAT_ID', 'type': ZWAVE_THERMOSTAT}
        ]

        for data_item in thermostat_db_data:
            cur.execute('SELECT * FROM '+data_item['table']+' WHERE '+data_item['room_col']+' = :room',
                        {'room': room_key})

            for item in cur.fetchall():
                thermostat = {'type': data_item['type'], 'id': item[data_item['id_col']]}
                thermostats.append(thermostat)

        print thermostats

        for thermostat in thermostats:
            heating_control(username, thermostat['type'], thermostat['id'], value, db)

    #set_context(username, SET_THERMOSTAT, {'room': room, 'value': value}, db)

    answer_data = [
        [get_okay(), [', '+username, ''], '.',
         ['', [' ', ['Die Heizung', 'Das Thermostat', 'Die Temperatur'], ' wurde ',
          ['', ['auf '+ value + ' Grad ']],
          ['angepasst', 'neu eingestellt', 'geändert'], '.']]]
    ]

    answer = generate_string(answer_data)

    return {'msg_speech': answer, 'msg_text': answer}

def set_dimmer(username, text, context, db):
    room = find_room(text, db)

    if room is not None:
        room_key = room['LOCATION']
    else:
        room_key = None

    devices = find_devices(text, room_key, db)
    return {'msg_speech':"Ok", 'msg_text':"Dimmer"}

def find_devices(text, location_key, db):
    devices = []

    with db:
        cur = db.cursor()

        #data => table_name, name_key, id_key, type_key, location_key
        data = [
            ['FUNKSTECKDOSEN','NAME','DEVICE',FUNKSTECKDOSE,'ROOM'],
            ['ZWAVE_SWITCHES','NAME','ID',ZWAVE_SWITCH,'LOCATION'],
            ['URL_SWITCH_BINARY','NAME','ID',URL_SWITCH,'LOCATION'],
            ['URL_TOGGLE','NAME','ID',URL_TOGGLE,'LOCATION'],
            ['PHILIPS_HUE_LIGHTS','NAME','ID',PHILIPS_HUE_LIGHT,'LOCATION'],
            ['URL_RGB_LIGHT','NAME','ID',URL_RGB_LIGHT,'LOCATION']
        ]

        words = text.split()

        for item in data:
            if location_key is None:
                cur.execute("SELECT * FROM "+item[0])
            else:
                param_array = {'location': location_key}
                cur.execute("SELECT * FROM "+item[0]+" WHERE "+item[4]+" = :location", param_array)

            for device in cur.fetchall():
                for word in words:
                    if word == device[item[1]].lower():
                        device_item = {'type': item[3], 'name': device[item[1]], 'id': device[item[2]]}
                        devices.append(device_item)

        '''for item in data:
            if location_key is None:
                cur.execute("SELECT * FROM "+item[0])
            else:
                param_array = {'location': location_key}
                cur.execute("SELECT * FROM "+item[0]+" WHERE "+item[4]+" = :location", param_array)

            for device in cur.fetchall():
                position = text.find(device[item[1]].lower())
                if position is not -1:
                    device_item = {'type': item[3], 'name': device[item[1]], 'id': device[item[2]]}
                    devices.append(device_item)'''

    return devices



#!/usr/bin/python
# -*- coding: utf-8 -*-
import json

import requests

from homevee.Manager.gateway import get_gateway
from homevee.db_utils import get_database_con


def get_rooms():
    result = {}

    gateway_data = get_gateway('Z-Wave', get_database_con())

    port = ""
    if (gateway_data['PORT'] != None and gateway_data['PORT'] != ""):
        port = ":" + str(gateway_data['PORT'])



    url = "http://" + gateway_data['KEY1'] + ':' + gateway_data['KEY2'] + '@' + gateway_data['IP'] + port + "/ZAutomation/api/v1/locations/"
    headers = {'content-type': 'application/json', 'Accept-Charset': 'UTF-8'}
    request = requests.get(url, headers=headers)
    data = request.content

    print data

    result = json.loads(data)

    return result